import Phaser from './libs/phaser'
import Helloworld from './states/helloworld'
import PreLoadState from './states/preLoadState'
import BootState from './states/bootState'
import NavState from './states/navState'
import GameState from './states/gameState'
import HistoryState from './states/historyState'

//重写Phaser.Game，记住一定在构造方法中调用super()方法，并传入相应的参数
class Game extends Phaser.Game {
    constructor() {
        /* 
            获取小游戏可用的屏幕尺寸
            （注意这里不一定是屏幕的实际尺寸，因为有的手机屏幕有些区域不可用，比如状态栏） 
        */
        let { windowWidth, windowHeight } = wx.getSystemInfoSync()

        //配置game的参数
        const conf = {
            width: windowWidth,
            height: windowHeight,
            canvas: canvas,
            renderer: Phaser.WEBGL,
            transparent: false,
            antialias: false,
        };
        //调用父类构造方法
        super(conf);

        //把游戏中所有的场景（也叫页面），添加到游戏中，类似react、vue中的路由
        this.state.add('Helloworld', Helloworld)
        this.state.add('BootState', BootState)
        this.state.add('PreLoadState', new PreLoadState(""))
        this.state.add('NavState', NavState);
        this.state.add('GameState', GameState);
        this.state.add('HistoryState', HistoryState);

        //启动第一个页面
        this.state.start('BootState')
    }
}

//初始化phaser程序
new Game();
