/*
 * @author 天天敲代码(lmy)
 * @description 我的记录页面，列表展示保存的歌曲弹奏记录，点击列表可以播放
 **/
import Phaser from '../libs/phaser';
import { ListView } from '../libs/list/index';
import soundSeek from '../util/soundSeek';

//全局创建屏幕可用宽高，便于使用
const { windowWidth, windowHeight } = wx.getSystemInfoSync();

export default class HistoryState extends Phaser.State {

    init(props) {
        console.log("启动 HistoryState");
    }

    shutdown() {
        //页面关闭，销毁listview
        this.listView.destroy();
    }

    create() {

        //背景颜色改为白色
        this.stage.backgroundColor = "#ffffff";

        let game = this.game;

        //列表头部的背景图片，宽高设置成宽度，高度为70
        let listheadImg = game.add.image(0, 0, "listhead");
        listheadImg.width = windowWidth;
        listheadImg.height = 70;

        //列表头部标题
        let headText = this.game.add.text(
            windowWidth / 2,
            50,
            "弹奏记录",
            { font: '20px Arial', fill: '#fff', align: "center" },
        );
        headText.anchor.set(0.5);

        //列表头部返回按钮，并添加回调方法
        let backImg = game.add.image(20, 45, "backw");
        backImg.anchor.set(0.5, 0.5);
        backImg.scale.set(0.25, 0.25);
        backImg.inputEnabled = true;
        backImg.events.onInputUp.add(() => {
            this.state.start("NavState")
        })

        //创建列表
        this.listView = new ListView(
            this.game,
            this.world,
            new Phaser.Rectangle(0, 70, windowWidth, windowHeight - 70),
            {
                direction: 'y',//设置列表纵向滚动
                searchForClicks: true //设置列表的每一个元素可以点击
            }
        );

        //获取存储的弹奏记录，若没有数据，则初始化为空数组
        let historyData = wx.getStorageSync("historyData") || [];

        //若没有弹奏的数据，可以使用这个来测试
        // historyData = [{ "name": "说好的幸福呢", "time": 1606364808946, "num": 1, "score": 11, "myMusic": [{ "key": "a3", "time": 1606364802210 }, { "key": "b3", "time": 1606364802678 }, { "key": "c4", "time": 1606364803101 }, { "key": "b3", "time": 1606364803582 }, { "key": "a3", "time": 1606364804033 }, { "key": "b3", "time": 1606364804658 }, { "key": "g3", "time": 1606364805071, "timeLong": 2812.5 }] }, { "name": "安静", "time": 1606364818102, "num": 2, "score": 15, "myMusic": [{ "key": "e4", "time": 1606364811673 }, { "key": "e4", "time": 1606364812108 }, { "key": "e4", "time": 1606364812542 }, { "key": "e4", "time": 1606364813005 }, { "key": "d4", "time": 1606364813434 }, { "key": "c4", "time": 1606364813866 }, { "key": "b3", "time": 1606364814358 }, { "key": "d4", "time": 1606364814889 }, { "key": "d4", "time": 1606364815418 }, { "key": "d4", "time": 1606364815820 }, { "key": "c4", "time": 1606364816243, "timeLong": 791.6666666666666 }] }, { "name": "生日歌", "time": 1606364827860, "num": 3, "score": 10, "myMusic": [{ "key": "g3", "time": 1606364819812 }, { "key": "g3", "time": 1606364820272 }, { "key": "a3", "time": 1606364820744 }, { "key": "g3", "time": 1606364821629 }, { "key": "c4", "time": 1606364822409 }, { "key": "b3", "time": 1606364823048, "timeLong": 3666.6666666666665 }] }, { "name": "勇气", "time": 1606364834315, "num": 4, "score": 19, "myMusic": [{ "key": "G3", "time": 1606364830379 }, { "key": "E3", "time": 1606364830797 }, { "key": "E3", "time": 1606364831236 }, { "key": "D3", "time": 1606364831630 }, { "key": "E3", "time": 1606364832054 }, { "key": "D3", "time": 1606364832394 }, { "key": "E3", "time": 1606364832834 }, { "key": "G3", "time": 1606364833108, "timeLong": 250 }] }, { "name": "晴天", "time": 1606364878968, "num": 5, "score": 28, "myMusic": [{ "key": "D3", "time": 1606364848436 }, { "key": "E3", "time": 1606364848883 }, { "key": "D3", "time": 1606364849256 }, { "key": "E3", "time": 1606364849691 }, { "key": "F3", "time": 1606364850056 }, { "key": "E3", "time": 1606364850466 }, { "key": "C3", "time": 1606364851643 }, { "key": "E3", "time": 1606364852296 }, { "key": "G3", "time": 1606364852883 }, { "key": "B3", "time": 1606364853313 }, { "key": "C4", "time": 1606364853748 }, { "key": "B3", "time": 1606364854255 }, { "key": "G3", "time": 1606364854798 }, { "key": "C3", "time": 1606364855264 }, { "key": "C3", "time": 1606364855682, "timeLong": 1116.6666666666667 }] }, { "name": "霓虹甜心", "time": 1606364893030, "num": 6, "score": 11, "myMusic": [{ "key": "G3", "time": 1606364884539 }, { "key": "A3", "time": 1606364884990 }, { "key": "G3", "time": 1606364885426 }, { "key": "G3", "time": 1606364886536 }, { "key": "E3", "time": 1606364886875 }, { "key": "G3", "time": 1606364887312 }, { "key": "E3", "time": 1606364888015, "timeLong": 3750 }] }, { "name": "蒲公英的约定", "time": 1606364902267, "num": 7, "score": 10, "myMusic": [{ "key": "e3", "time": 1606364895490 }, { "key": "f3", "time": 1606364895997 }, { "key": "g3", "time": 1606364896492 }, { "key": "g3", "time": 1606364897441 }, { "key": "g3", "time": 1606364897964 }, { "key": "g3", "time": 1606364898434, "timeLong": 750 }] }, { "name": "小星星", "time": 1606364914932, "num": 8, "score": 11, "myMusic": [{ "key": "c3", "time": 1606364908199 }, { "key": "c3", "time": 1606364908764 }, { "key": "g3", "time": 1606364909230 }, { "key": "g3", "time": 1606364909697 }, { "key": "a3", "time": 1606364910157 }, { "key": "a3", "time": 1606364910640 }, { "key": "g3", "time": 1606364911051, "timeLong": 1583.3333333333333 }] },]

        //根据数据，循环生成列表的展示对象
        for (var i = 0; i < historyData.length; i++) {
            //倒序展示列表的数据
            let item = historyData[historyData.length - 1 - i];

            //graphics生成每条展示的背景为白色，最下面是一条灰色的线条
            let g = this.game.add.graphics(0, 0);
            g.beginFill(0xffffff).drawRect(0, 0, windowWidth, 70);
            g.lineStyle(1, 0xededed);
            g.moveTo(0, 69);
            g.lineTo(windowWidth, 69);
            //graphics创建一个图片对象作为一个列表的条目
            let bgImg = this.game.add.image(0, 0, g.generateTexture());
            g.destroy();

            //为每一个条目添加点击事件，点击后可以播放弹奏的记录
            bgImg.inputEnabled = true;
            bgImg.events.onInputUp.add(() => {
                console.log("play music");
                this.play(bgImg);
            });

            //把每一条的曲子数据挂载到bgImg上，方便数据的使用
            bgImg.songData = item;

            //列表元素中的最左边的图标
            let listenImg = this.game.make.image(6, 35, 'listen');
            listenImg.anchor.set(0, 0.5);
            listenImg.scale.set(0.2, 0.2);
            bgImg.addChild(listenImg);


            //列表元素中曲子的名字
            let nameText = this.game.make.text(36, 8,
                item.name,
                { font: '20px ', fill: '#000' },
            );
            nameText.anchor.set(0, 0);
            bgImg.addChild(nameText);

            //列表元素中歌曲是第几关的文字
            let guanText = this.game.make.text(nameText.width + 40, 12,
                "(第" + item.num + "关)",
                { font: '14px ', fill: '#888888' },
            );
            guanText.anchor.set(0, 0);
            bgImg.addChild(guanText);

            //列表元素中的曲子保存的时间文字
            let timeText = this.game.make.text(40, 68,
                "时间:" + this.formatDate(item.time),
                { font: '12px ', fill: '#000' },
            );
            timeText.anchor.set(0, 1);
            bgImg.addChild(timeText);

            //列表元素中的曲子的得分
            let scoreText = this.game.make.text(windowWidth - 15, 35,
                item.score + "分",
                { font: '20px ', fill: '#66b0be' },
            );
            scoreText.anchor.set(1, 0.5);
            bgImg.addChild(scoreText);

            //把每个元素添加到列表对象中
            this.listView.add(bgImg);
        }
    }

    //播放每一个存储的弹奏记录,参数为listview中的元素
    play(listItem) {

        //用于记录播放到哪个音符
        let playInfo = { index: 0 }
        //播放进度计算
        this.playProgress(listItem, playInfo);

        //播放弹奏的曲子,播放第一个音符时候，第三个参数为空
        this.playMusic(listItem, playInfo, null);

    }


    /* 
    *计算曲子播放的进度，以及处理终止当前的曲子，并播放下一个
    *listItem为liveView中的元素
    *playInfo用于记录播放到哪个音符
    */
    playProgress(listItem, playInfo) {

        //如果当前正在播放某个弹奏的记录，则终止这个正在播放的音频
        if (this.listItem) {
            //取消他的进度显示
            this.listItem.removeChild(this.listItem.progress);

            //停止计时和音频播放的超时定时器
            this.time.events.remove(this.buildLoop);
            this.time.events.remove(this.timeout);

            //销毁正在播放的音频
            this.currentAudio.destroy();
        }
        //在要播放的listItem挂载到this上，播放完毕或者中途播放其他的，把这个正在播放的去除
        this.listItem = listItem;

        let songData = listItem.songData;

        //获取所有音符的数据
        let myMusic = songData.myMusic;

        //计算曲子弹奏的总时长=最后一个音符弹奏的时间-第一个音符的弹奏时间+最后一个音符的时值
        let time = myMusic[myMusic.length - 1].time - myMusic[0].time
            + myMusic[myMusic.length - 1].timeLong


        //添加播放进度的显示对象，就是一个text对象
        let progress = this.game.make.text(
            windowWidth - 70, 35,
            "00:00:00/" + this.formatDuring(time),
            { font: '12px ', fill: '#000' }
        );
        listItem.progress = progress;
        progress.anchor.set(1, 0.5);
        listItem.addChild(progress);

        //弹奏记录的开始播放时间，用于记录并计算播放的时间进度
        let playtime = Date.now();

        //计时循环定时器
        let buildLoop = this.time.events.loop(200, () => {

            //曲子的播放进度，每200毫秒执行一次，由于执行时间在不够精确，所以进度的计算
            //用当前的时间减去this.playtime，这个时间间隔，只要<=1000毫秒都可以
            let difftime = Date.now() - playtime;

            //在转换时间格式的时候，舍去秒之后的小数，formatDuring第二个参数为true
            let showTime = this.formatDuring(difftime, true);
            //如果播放时间大于曲子的总时长后，时间的进度就等于，歌曲总时长，总时长的计算为小数之后进1，formatDuring为默认的false
            //这里这么做的原因是：防止曲子播放的实时进度，大于曲子的总时长
            if (difftime > time) {
                showTime = this.formatDuring(time);
            }
            progress.text = showTime + "/" + this.formatDuring(time);

            //曲子播放完毕1秒后，移除这个这个歌曲的播放进度
            if (difftime > time + 1000) {
                this.time.events.remove(buildLoop);
                listItem.removeChild(listItem.progress);
                //播放完毕，把this.listItem置为空
                this.listItem = null;
            }
            this.buildLoop = buildLoop;
        });
    }

    /* 
    *播放弹奏的曲子
    *listItem为liveView中的元素
    *playInfo用于记录播放到哪个音符
    *previousAudio为上一个播放的音频对象
    */
    playMusic(listItem, playInfo, previousAudio) {

        //获取播放的音符数据
        let songData = listItem.songData;
        let myMusic = songData.myMusic;

        //创建播放的音符的音频
        let key = myMusic[playInfo.index].key;
        let audio = wx.createInnerAudioContext();

        //把当前播放的音频挂在到this上，在中断时候，方便销毁
        this.currentAudio = audio;

        //取到当前音符属于那一组
        let groupNum = key.substring(key.length - 1);
        //由于变量不能是开头#、♭，为了防止程序有问题，特意把#转成h,把♭转成l
        let soundName = key.replace("#", "h").replace("♭", "l");
        audio.src = "assets/audio/" + groupNum + "/" + soundName + ".mp3";

        //由于音频创建完成，需要加载音频数据，不能立即播放，onCanplay回到之后音频就一定可以播放了
        //如果疑问游戏页面为什么不这样做，因为游戏页面不是创建完音频毕立即就播放
        let onCanplay = audio.onCanplay(() => {
            //一次音频onCanplay会回调多次，所以回调完毕后直接，取消监听
            audio.offCanplay(onCanplay);
            //播放音符对应的音频，从校正后的开始时间，开始播放
            audio.seek(soundSeek[soundName.toUpperCase()]);
            audio.play();

            //200毫秒后消除上一个音符的音频，设置200毫秒后消除的原因是有延长音效果，以免有断断续续的感觉
            if (previousAudio) {
                this.time.events.add(200, () => {
                    previousAudio.destroy();
                });
            }

            //两种状态，是不是最后一个音符
            if (playInfo.index < myMusic.length - 1) {
                //如果不是最后一个音符，计算当前音符时值=下一个音符的弹奏时间戳 减去 当前音符的弹奏时间
                let time = myMusic[playInfo.index + 1].time - myMusic[playInfo.index].time;

                //播放的音符加1，移到下一个
                playInfo.index = playInfo.index + 1;

                //设置一个超时定时器，递归调用播放下一个音符，超时时间，就是音符的的时值
                this.timeout = this.time.events.add(time, () => {
                    this.playMusic(listItem, playInfo, audio);
                });
            } else {
                //最后一个音频的播放时间，为其timeLong
                let time = myMusic[playInfo.index].timeLong;

                //超时定时器销毁最后一个音符
                this.timeout = this.time.events.add(time,
                    () => {
                        audio.destroy();
                        this.time.events.remove(this.timeout);
                    });
            }
        });
    }

    //把时间戳转换成时分秒（格式：01:09:12）
    formatDuring(time, isFloor = false) {
        var hours = parseInt((time % (1000 * 60 * 60 * 24)) / (1000 * 60 * 60));
        if (hours < 10) {
            hours = "0" + hours;
        }
        var minutes = parseInt((time % (1000 * 60 * 60)) / (1000 * 60));
        if (minutes < 10) {
            minutes = "0" + minutes;
        }

        var seconds = Math.ceil((time % (1000 * 60)) / 1000);
        if (isFloor) {
            seconds = Math.floor((time % (1000 * 60)) / 1000);
        }
        if (seconds < 10) {
            seconds = "0" + seconds;
        }
        return hours + ":" + minutes + ":" + seconds;
    }

    //把时间戳格式化成年月日时分秒(格式：2020-08-16 10:08:18)
    formatDate(time) {
        var date = new Date(time);
        var YY = date.getFullYear() + '-';
        var MM = (date.getMonth() + 1 < 10 ? '0' + (date.getMonth() + 1) : date.getMonth() + 1) + '-';
        var DD = (date.getDate() < 10 ? '0' + (date.getDate()) : date.getDate());
        var hh = (date.getHours() < 10 ? '0' + date.getHours() : date.getHours()) + ':';
        var mm = (date.getMinutes() < 10 ? '0' + date.getMinutes() : date.getMinutes()) + ':';
        var ss = (date.getSeconds() < 10 ? '0' + date.getSeconds() : date.getSeconds());
        return YY + MM + DD + " " + hh + mm + ss;
    }
}